# File rvsao/Util/getspec.x
# July 8, 1996
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After Gerard Kriss, Johns Hopkins University

# Copyright(c) 1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
#  GETSPEC opens the image specified by specfile and returns pointers
#  to the data and the image descriptor.  Data is shared in "rvsao.com"
 
include	"rvsao.h"
include	"shdr.h"
 
procedure getspec (specfile, mspec, spectrum, specim, specsh)
 
char	specfile[ARB]	# Data file name
int	mspec		# Number of spectrum to read from multispec file
pointer	spectrum	# Spectrum data (returned)
pointer	specim		# Image header structure (returned)
pointer	specsh		# Spectrum header structure (returned)
 
int	npix,ap
 
real	compbcv()
double	dlog10()
double	bcvel,hcvel,dindef
int	mode
int	imaccess()
 
include	"rvsao.com"
include	"getim.com"
 
begin
	dindef = INDEF

	if (debug) {
	    call printf ("GETSPEC: %s %d\n")
		call pargstr (specfile)
		call pargi (mspec)
	    }

# Check spectrum file for writeability before opening it
	mode = READ_WRITE
	if (imaccess (specfile, READ_WRITE) == NO) {
	    call eprintf ("GETSPEC: cannot write to %s; not saving results\n")
		call pargstr (specfile)
            mode = READ_ONLY
	    }

# Read spectrum
	call getimage (specfile,mspec,spectrum,specim,specsh,npix,
		       specname, mode, debug)
	if (specim == ERR) {
	    call printf ("GETSPEC:  Error reading %s[%d]\n")
		call pargstr (specfile)
		call pargi (mspec)
	    return
	    }

# Set up spectrum line and aperture in specid
	if (mspec > 0) {
	    ap = AP(specsh)
	    if (ap != mspec) {
		call sprintf (specid,SZ_PATHNAME,"%s[%d ap%d]")
		    call pargstr (specfile)
		    call pargi (mspec)
		    call pargi (ap)
		}
	    else {
		call sprintf (specid,SZ_PATHNAME,"%s[%d]")
		    call pargstr (specfile)
		    call pargi (mspec)
		}
	    }
	else
	    call strcpy (specfile,specid,SZ_PATHNAME)

 
# Set number of spectrum pixels in labelled common
	specpix = npix
	if (debug) {
	    call printf ("GETSPEC: %s  %d pixels read\n")
		call pargstr (specname)
		call pargi (specpix)
	    }

# Compute log wavelength image of spectrum
	specdc = DC(specsh)
        call imgipar (specim,"DC-FLAG ",specdc)
	if (W0(specsh) > 0.)
	    spstrt = dlog10(W0(specsh))
	else {
	    call printf ("GETSPEC: Illegal starting wavelength %.2f\n")
		call pargd (W0(specsh))
	    if (W1(specsh) > 0. && WP(specsh) != 0.)
		W0(specsh) = W1(specsh) + double (specpix-1) * WP(specsh)
	    else {
		call close_image (specim, specsh)
		specim = ERR
		return
		}
	    }
	if (W1(specsh) > 0.)
	    spfnsh = dlog10(W1(specsh))
	else {
	    call printf ("GETSPEC: Illegal ending wavelength %.2f\n")
		call pargd (W1(specsh))
	    if (W0(specsh) > 0. && WP(specsh) != 0.)
		W1(specsh) = W0(specsh) + double (specpix-1) * WP(specsh)
	    else {
		call close_image (specim, specsh)
		specim = ERR
		return
		}
	    }
	if (debug) call printf ("GETSPEC: about to read velocity info\n")

# Set up pixel <-> wavelength transformation
	call wcs_set (specsh)

# Get radial velocity and error
	vel = VEL(specsh)
	spechcv = 0.d0
	bcvel = dindef
	hcvel = dindef
	call vrhead (mspec, specim)
	call xcrhead (mspec, specim)
	call emrhead (mspec, specim)
	if (mspec <= 0 || (mspec == 1 && spechcv == dindef)) {
	    call imgdpar (specim, "BCV", bcvel)
	    call imgdpar (specim, "HCV", hcvel)
	    }
	else
	    bcvel = spechcv
	if (debug) {
	    call printf ("GETSPEC: hcvel = %.3f, bcvel = %.3f, svcor = %d\n")
		call pargd (hcvel)
		call pargd (bcvel)
		call pargi (svcor)
	    }

# Calculate heliocentric velocity, or use one from file
	switch (svcor) {
	    case NONE:
		spechcv = 0.d0
	    case HCV:
		spechcv = compbcv (specim,0,debug)
	    case BCV:
		spechcv = compbcv (specim,1,debug)
	    case FHCV:
		if (hcvel != dindef) {
		    spechcv = hcvel
		    specvb = FALSE
		    }
		else if (bcvel != dindef) {
		    spechcv = bcvel
		    specvb = TRUE
		    }
		else {
		    spechcv = compbcv (specim,0,debug)
		    if (spechcv == dindef)
			spechcv = 0.d0
		    specvb = FALSE
		    }
	    case FBCV:
		if (bcvel != dindef) {
		    spechcv = bcvel
		    specvb = TRUE
		    }
		else if (hcvel != dindef) {
		    spechcv = hcvel
		    specvb = FALSE
		    }
		else {
		    spechcv = compbcv (specim,1,debug)
		    if (spechcv == dindef)
			spechcv = 0.d0
		    specvb = TRUE
		    }
	    }

# Debugging information about this spectrum
	if (debug) {
	    call printf ("GETSPEC: vel: %.3f hcv: %.3f bcv? %b\n")
		call pargd (vel)
		call pargd (spechcv)
		call pargb (specvb)
	    call printf ("GETSPEC: lambda %9.4f - %9.4f by %9.4f")
		call pargd (W0(specsh))
		call pargd (W1(specsh))
		call pargd (WP(specsh))
	    if (DC(specsh) != DCLOG) {
		call printf (" = %d points\n")
		    call pargi (specpix)
		}
	    else
		call printf ("\n")
	    dlogw = (spfnsh - spstrt) / double (specpix - 1)
	    call printf ("GETSPEC: log lambda %9.7f - %9.7f by %9.7f")
		call pargd (spstrt)
		call pargd (spfnsh)
		call pargd (dlogw)
	    if (DC(specsh) == DCLOG) {
		call printf (" = %d points\n")
		    call pargi (specpix)
		}
	    else
		call printf ("\n")
	    }

end
 
# June	1987	Gerard Kriss
# Sept	1989	Doug Mink--add parameter for 2-D files
# July	1990	Doug Mink--add HCV
# Sept	1990	Doug Mink--add velocity parameters
# Oct 	1990	Doug Mink--add hcv or bcv from file
# June	1991	Doug Mink--open image READ_WRITE when needed

# Feb 14 1992	Call new getimage which can handle multispec files
# Apr 20 1992	Pass mspec as argument

# May 24 1993	Add MWCS pixel<->wavelength transformation
# May 25 1993	Get radial velocity information directly from the image header
# Jun  2 1993	Use default velocity from mwcs
# Jun  3 1993	Use onedspec spectrum header structure
# Jun 16 1993	Fix use of onedspec spectrum header
# Jul  7 1993	Add spectrum header to getimage arguments
# Nov 23 1993	Read cross-correlation velocity error BEFORE R-value
# Dec  1 1993	Fix bug setting multispec keyword names

# Mar  8 1994	Fix bug listing wavelength increment
# Apr 12 1994	Return MWCS header pointer
# May 23 1994	Use header-save flag from labelled common
# Jun 15 1994	Remove unused variables
# Jun 22 1994	Save all previous velocity results
# Jun 23 1994	Keep MWCS pointer in SHDR pointer
# Aug  3 1994	Change common and header from fquot to rvsao
# Dec  7 1994	Read velocity information using XCRHEAD and EMRHEAD
# Dec  8 1994	Initialize mwcs here instead of in main programs

# Mar 28 1995	Return name of file if error occurs
# May 10 1995	Print velocities to meters/sec
# May 26 1995	Handle barycentric correction for multispec spectra
# Jun 19 1995	Set ID string with file, line, and aperture (if not =line)
# Jul  7 1995	Fix bug switching between BCV and HCV header keywords
# Jul 13 1995	Add DEBUG to COMPBCV calls
# Aug  4 1995	If BCV and HCV aren't in header, compute BCV or HCV
# Sep 21 1995	Open file read_write unless it cannot be written

# Jul  8 1996	Pass debugging flag to getimage
